
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

#include <glib-object.h>
#include <gst/tag/tag-prelude.h>

G_BEGIN_DECLS

/* enumerations from "gsttagdemux.h" */

GST_TAG_API
GType gst_tag_demux_result_get_type (void);
#define GST_TYPE_TAG_DEMUX_RESULT (gst_tag_demux_result_get_type())

/* enumerations from "tag.h" */

GST_TAG_API
GType gst_tag_image_type_get_type (void);
#define GST_TYPE_TAG_IMAGE_TYPE (gst_tag_image_type_get_type())

GST_TAG_API
GType gst_tag_license_flags_get_type (void);
#define GST_TYPE_TAG_LICENSE_FLAGS (gst_tag_license_flags_get_type())

G_END_DECLS

/* Generated data ends here */

